from typing import List
from pydantic import Field
from .base import BaseCommand

class TransferCommand(BaseCommand):
    stepType: str = Field(default="moveLiquid", description="The type of the step")
    stepName: str = Field(default="Pipetting", description="Name of the step")
    
    # Basic transfer settings
    path: str = Field(default="single", description="Path type for liquid transfer")
    volume: float = Field(..., description="Volume to transfer")
    pipette: str = Field(..., description="ID of pipette to use")
    tipRack: str = Field(..., description="Tip rack to use")
    changeTip: str = Field(..., description="When to change tips")
    
    # Source and destination
    aspirate_wells: List[str] = Field(..., description="Wells to aspirate from")
    dispense_wells: List[str] = Field(..., description="Wells to dispense to")
    aspirate_labware: str = Field(..., description="Labware to aspirate from")
    dispense_labware: str = Field(..., description="Labware to dispense to")
    
    # Tip handling
    dropTip_location: str = Field(..., description="Location to drop tip")
    preWetTip: bool = Field(default=False, description="Whether to pre-wet tip")
    
    # Flow control
    blowout_checkbox: bool = Field(default=False, description="Whether to perform blowout")
    disposalVolume_volume: float = Field(default=30, description="Disposal volume")
    disposalVolume_checkbox: bool = Field(default=False, description="Whether to use disposal volume")
    
    # Air gap settings
    aspirate_airGap_volume: float = Field(default=30, description="Air gap volume for aspiration")
    aspirate_airGap_checkbox: bool = Field(default=False, description="Whether to use air gap during aspiration")
    dispense_airGap_volume: float = Field(default=30, description="Air gap volume for dispensing")
    dispense_airGap_checkbox: bool = Field(default=False, description="Whether to use air gap during dispensing")
    
    # Mixing settings
    aspirate_mix_checkbox: bool = Field(default=False, description="Whether to mix while aspirating")
    dispense_mix_checkbox: bool = Field(default=False, description="Whether to mix while dispensing")
    
    # Delay settings
    aspirate_delay_seconds: int = Field(default=1, description="Delay after aspiration")
    aspirate_delay_checkbox: bool = Field(default=False, description="Whether to delay after aspiration")
    dispense_delay_seconds: int = Field(default=1, description="Delay after dispensing")
    dispense_delay_checkbox: bool = Field(default=False, description="Whether to delay after dispensing")
    
    # Well ordering
    aspirate_wells_grouped: bool = Field(default=False, description="Whether wells are grouped")
    aspirate_detect_checkbox: bool = Field(default=False, description="Whether to detect liquid level")
    aspirate_wellOrder_first: str = Field(default="t2b", description="First well ordering direction for aspiration")
    aspirate_wellOrder_second: str = Field(default="l2r", description="Second well ordering direction for aspiration")
    dispense_wellOrder_first: str = Field(default="t2b", description="First well ordering direction for dispensing")
    dispense_wellOrder_second: str = Field(default="l2r", description="Second well ordering direction for dispensing")
    
    # Touch tip settings
    aspirate_touchTip_checkbox: bool = Field(default=False, description="Whether to touch tip after aspiration")
    dispense_touchTip_checkbox: bool = Field(default=False, description="Whether to touch tip after dispensing")

class MixCommand(BaseCommand):
    stepType: str = Field(default="mix", description="The type of the step")
    stepName: str = Field(default="Mix", description="Name of the step")

    # Basic mix settings
    times: int = Field(..., description="Number of mix repetitions")
    volume: float = Field(..., description="Volume to mix")
    wells: List[str] = Field(..., description="Wells to mix in")
    labware: str = Field(..., description="Labware containing wells")
    
    # Pipette settings
    pipette: str = Field(..., description="ID of pipette to use")
    tipRack: str = Field(..., description="Tip rack to use")
    changeTip: str = Field(..., description="When to change tips")
    
    # Flow settings
    aspirate_flowRate: float = Field(..., description="Aspirate flow rate")
    dispense_flowRate: float = Field(..., description="Dispense flow rate")
    blowout_checkbox: bool = Field(default=False, description="Whether to perform blowout")
    
    # Additional settings
    dropTip_location: str = Field(..., description="Location to drop tip")
    
    # Well ordering
    mix_wellOrder_first: str = Field(default="t2b", description="First well ordering direction")
    mix_wellOrder_second: str = Field(default="l2r", description="Second well ordering direction")
    
    # Touch tip and delay settings
    mix_touchTip_checkbox: bool = Field(default=False, description="Whether to touch tip after mixing")
    aspirate_delay_seconds: int = Field(default=1, description="Delay after aspiration")
    dispense_delay_seconds: int = Field(default=1, description="Delay after dispensing")
    aspirate_delay_checkbox: bool = Field(default=False, description="Whether to delay after aspiration")
    dispense_delay_checkbox: bool = Field(default=False, description="Whether to delay after dispensing") 